/* Copyright 1999 by Herve Regad-Pellagru, E-mail: herve.regadpellagru@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <x10_api.h>

#include "common.h"
#include "proto.h"


#define ERROR_CMD {error_cmd(); return;}

extern int sock;
extern struct x10_status_t *x10_status;
extern FILE *log_file;
extern int listen_mode;

static char *OK="OK\n";
static char string[MAXBUF];


/* This handles incorrect commands received */
void error_cmd(void)
{
  strcpy(string,"input ERROR\n");
  SEND_STRING(string);
  SEND_STRING(OK);
}


/* This is the send command. */
void do_cmd_send(void)
{
  char *house,*tmp_char;
  int unit,command,arg;

  if(!(house = strtok(NULL," ")))
    ERROR_CMD;

  if(!(tmp_char = strtok(NULL," ")))
    ERROR_CMD;

  unit = atoi(tmp_char);

  if(!(tmp_char = strtok(NULL," ")))
    ERROR_CMD;

  command = string2cmd(tmp_char);

  if(command == -1)
    ERROR_CMD;

  if(!(tmp_char = strtok(NULL," ")))
    ERROR_CMD;

  arg = atoi(tmp_char);

  if(x10_send_cmd(house[0],unit,command,arg,x10_status))
    ERROR_CMD;

  sprintf(string,"SEND %c%d %s (%d)\n",house[0],unit,cmd2string(command),
	  arg);

  SENDLOG_STRING(string);
  SEND_STRING(OK);
}


/* This is the status command. */
void do_cmd_status(void)
{
  char *house,*tmp_char;
  int unit;
  struct x10_object_t module;

  if(!(house = strtok(NULL," ")))
    ERROR_CMD;

  if(!(tmp_char = strtok(NULL," ")))
    ERROR_CMD;

  unit = atoi(tmp_char);
  module=x10_status->objects[house[0]-'A'][unit-1];

  if(house[0]<'A' || house[0]>'P' || unit<1 || unit>16)
    ERROR_CMD;

  sprintf(string,"STATUS of %c%d is %s since %s\n",house[0],unit,
	  status2string(module.status),print_ltimestamp(module.timestamp));
  SEND_STRING(string);
  SEND_STRING(OK);
}


/* This is the status_all command. */
void do_cmd_status_all(void)
{
  struct x10_object_t module;
  int i,j;

  for(i=0;i<16;i++)
    for(j=0;j<16;j++)
      {
	module=x10_status->objects[i][j];
	sprintf(string,"STATUS of %c%d is %s since %s\n",'A'+i,j+1,
		status2string(module.status),
		print_ltimestamp(module.timestamp));
	SEND_STRING(string);
      }

  SEND_STRING(OK);
}



/* This is the log command. */
void do_cmd_log(void)
{
  long previous_pos;

  previous_pos = ftell(log_file);
  fseek(log_file,0L,SEEK_SET);

  while(fgets(string,MAXBUF,log_file))
    SEND_STRING(string);

  SEND_STRING(OK);
  fseek(log_file,previous_pos,SEEK_SET);
}


/* This is the listen command. */
void do_cmd_listen(void)
{
  listen_mode=1;
  SEND_STRING(OK);
}


/* This is the no_listen command. */
void do_cmd_nolisten(void)
{
  listen_mode=0;
  SEND_STRING(OK);
}


/* This is the help command. */
void do_cmd_help(void)
{
  strcpy(string,"x10_server commands are:\n");
  SEND_STRING(string);
  strcpy(string,"- send house unit command arg: sends a command to a module\n");
  SEND_STRING(string);
  strcpy(string,"  (ex: send A 4 X10_CMD_ON 0, send A 5 X10_CMD_BRIGHT 12)\n");
  SEND_STRING(string);
  strcpy(string,"- status house unit: gets the status for a module\n");
  SEND_STRING(string);
  strcpy(string,"- status_all: gets the status for all modules\n");
  SEND_STRING(string);
  strcpy(string,"- log: gets the log of the server\n");
  SEND_STRING(string);
  strcpy(string,"- listen: toggles ON the listen mode\n");
  SEND_STRING(string);
  strcpy(string,"- nolisten: toggles OFF the listen mode\n");
  SEND_STRING(string);
  strcpy(string,"- help: this screen\n");
  SEND_STRING(string);
  SEND_STRING(OK);
}
